#'Read requested dates from recipe and return array of file dates to be loaded
#'
#'The purpose of this function is to read the recipe configuration data for 
#'Auto-S2S workflows, retrieve the start and end dates for the hindcast and
#'the forecast dates, and return two arrays containing the requested dates, to
#'be passed to the Start() call. If no fcst date is provided, it returns an
#'empty object.
#'
#'@param recipe Auto-S2S configuration recipe as returned by read_yaml()
#'@param logger object of class logger containing log output file information
#'
#'@return a list of two arrays containing file dates for hcst and fcst
#'
#'@export

library(lubridate)
 
dates2load <- function(recipe, logger) {

  temp_horizon <- tolower(recipe$Analysis$Horizon)
  temp_freq <- recipe$Analysis$Variables$freq
  recipe <- recipe$Analysis$Time
  
  if (temp_horizon == "seasonal") {
    # hcst dates
    file_dates <- paste0(strtoi(recipe$hcst_start):strtoi(recipe$hcst_end),
                         recipe$sdate)
  
    if (temp_freq == "monthly_mean") {
      file_dates <- .add_dims(file_dates, "seasonal")
    }
  # fcst dates (if fcst_year empty it creates an empty object)
    if (!is.null(recipe$fcst_year)) {
      file_dates.fcst <- paste0(recipe$fcst_year, recipe$sdate)
      if (temp_freq == "monthly_mean") {
        file_dates.fcst <- .add_dims(file_dates.fcst, "seasonal")
      }
    } else {
      file_dates.fcst <- NULL
      info(logger,
         paste("fcst_year empty in the recipe, creating empty fcst object..."))
    }
  } else if (temp_horizon == "subseasonal") {
    if (is.null(recipe$fcst_year)) {
      stop("Error: A reference year (fcst_syear) must be provided for ",
	   "subseasonal forecast verification.")
    } else {
      ## TODO: Deal with February 29
      fcst_sdate <- paste0(recipe$fcst_year, recipe$sdate)
      sday_dates <- get.file_dates(fcst_sdate)
      file_dates <- do.call(paste0, expand.grid(sday_dates,
						strtoi(recipe$hcst_start)
						:strtoi(recipe$hcst_end)))
      # Reformat the dates to YYYYMMDD
      file_dates <- strptime(as.character(file_dates), '%m%d%Y')
      file_dates <- format(file_dates, '%Y%m%d')
      file_dates.fcst <- paste0(recipe$fcst_year, sday_dates)
      # Add dimensions to hcst and fcst dates
      file_dates <- .add_dims(file_dates,
			      temp_horizon = temp_horizon,
			      days = ncol(sday_dates),
			      weeks = nrow(sday_dates))
      file_dates.fcst <- .add_dims(file_dates.fcst,
				   temp_horizon = temp_horizon,
				   days = ncol(sday_dates),
				   weeks = nrow(sday_dates))
    }
  }
  return(list(hcst = file_dates, fcst = file_dates.fcst))
  ## TODO: document header of fun
}

# adds the correspondent dims to each sdate array
.add_dims <- function(data, temp_horizon = "seasonal", days = 1, weeks = 1) {
  if (temp_horizon == "seasonal") {
    default_dims <- c(sday = days, sweek = weeks, syear = length(data))
  } else if (temp_horizon == "subseasonal") {
    default_dims <- c(days, weeks, syear = length(data)/(days[[1]]*weeks[[1]]))
  }

  default_dims[names(dim(data))] <- dim(data)
  dim(data) <- default_dims
  return(data)
}

# Gets the corresponding dates or indices according
# to the sdate/leadtimes requested in the recipe
# 
# The leadtimes are defined by months
# Ex. 20201101 with leadtimes 1-4 corresponds to 
# the forecasting times covering December to March

get_timeidx <- function(sdates, ltmin, ltmax, 
                        time_freq="monthly_mean") {

  if (time_freq == "daily_mean") {

    sdates <- ymd(sdates)
    idx_min <- sdates + months(ltmin - 1)
    idx_max <- sdates + months(ltmax) - days(1)

    indxs <- array(numeric(), c(file_date=length(sdates),
                                time = (as.integer(idx_max[1]-idx_min[1]+1))
                                #syear = length(sdates),
                                #sday = 1, sweek = 1,
                                ))

    for (sdate in 1:length(sdates)) {
      days <- seq(idx_min[sdate], idx_max[sdate], by='days')
      indxs[sdate,] <-  days[!(format(days, "%m%d") == "0229")]
    }
    indxs <- as.POSIXct(indxs*86400, 
                        tz = 'UTC', origin = '1970-01-01')
    lubridate::hour(indxs) <- 12 
    lubridate::minute(indxs) <- 00
    dim(indxs) <- c(file_date=length(sdates),
                    time=(as.integer(idx_max[1]-idx_min[1])+1))

  } else if (time_freq == "weekly_mean") {
    idx_min <- 1
    idx_max <- 4
    indxs <- indices(idx_min:idx_max)

  } else if (time_freq == "monthly_mean") {
  
    idx_min <- ltmin
    idx_max <- ltmax
    indxs <- indices(idx_min:idx_max)
  }

  # TODO: 6 hourly case
  #idx1 <- (sdates + months(ltmin-1) - sdates)*4
  #idx2 <- idx1 + ndays*4 - 1

  return(indxs)
}

## TODO: Adapt Lluís' function to compute subseasonal startdates
#-------------------------------------------------------------------
get.file_dates <- function(fcst_sdate) {
  # Inputs: fcst.sdate - str in YYYYMMDD format  
  # This function generates the sdates path to retrieve the hcst
  # files. Outputs a multi-dimensional array with dims:
  # (sweek, sday) containing the character string needed.


  fcst_sdate <- as.Date(fcst_sdate, "%Y%m%d")
  sdates <- seq(fcst_sdate - 14, fcst_sdate + 14, "days")
  sdates <- sdates[which(format(sdates, "%a") %in% c("Mon", "Thu"))]

  file_dates <- array(numeric(), c(0, 3))
  for (sdate in as.list(sdates)) {
    # makes sure of giving a: [M T M || T M T] combination
    if (format(sdate, "%a") == "Thu") {
      hcst.sdays <- c(sdate - 3, sdate, sdate + 4, recursive=T)
    } else {
      hcst.sdays <- c(sdate - 4, sdate, sdate + 3, recursive=T)
    }

    hcst.sdays <- format(as.Date(hcst.sdays, "%Y%m%d"), "%m%d")
    #names(dim(hcst.sdays)) <- c('sday')
    file_dates <- abind(file_dates, hcst.sdays, along=1)
    #file_dates.fcst <- abind(file_dates.fcst, sdays.file_dates.fcst, along=1)
  }
  names(dim(file_dates)) <- c('sweek', 'sday')
  return(file_dates)
}
